'''
Simulate Producer-Consumer Model
'''
import random 
import asyncio
import json
from datetime import datetime

all_foods = ['Beef Goulash', 'Chicken Parmesan Pasta', 'Air Fryer Steak', 'Pumpkin Soup', 'Fried Rice', 'Apple Pie', 'Favourite Lasagne', 'Thai Beef Salad']

def do_queue(queue, role, item="") :
    try:
        ts = datetime.now().strftime("%H:%M:%S")
        if item == "" :
            item = queue.get_nowait() # raise exception if empty
            print(f"\n{ts} {role} - **{item}** TAKEN AWAY")
        else :
            queue.put_nowait(item)    # raise exception if full
            print(f"\n{ts} {role}   - **{item}** READY")
            
    except asyncio.QueueEmpty:
        print(f"\n{ts} {role} - ! Food Queue Empty !!!")
        
    except asyncio.QueueFull:
        print(f"\n{ts} {role}   - ! Food Queue Full !!!")
    
# Asynchronous routines
async def producer(queue):
    while True:
        do_queue(queue, "producer", random.choice(all_foods))
        await asyncio.sleep(3)   

async def consumer_1(queue):
    while True:
        do_queue(queue, "consumer_1")     
        await asyncio.sleep(random.randint(3,10))   

async def consumer_2(queue):
    while True:
        do_queue(queue, "consumer_2")       
        await asyncio.sleep(random.randint(3,10)) 
        
async def main():
    queue = asyncio.Queue(3) # max size
    task0 = asyncio.create_task(producer(queue))
    task1 = asyncio.create_task(consumer_1(queue))
    task2 = asyncio.create_task(consumer_2(queue))
    await task0
    await task1
    await task2

asyncio.run(main())